<?php

/**
 * Classe per la creazione di form html.
 * Costruisce il form a partire da una definizione in XML, mette a disposizione
 * funzionalita' per la validazione dei dati (javascript e php), precompilazione
 * dei campi, popolamento dei campi select da database, modifica
 * run-time delle label ed action del form. Consente di replicare porzioni o
 * l'intero form N volte senza duplicazioni nella definizione XML.
 *
 * Version 1.1	- aggiunto plugin TinyMCE
 * 				- caricamento dei JS solo per le estensioni utilizzate
 *
 * @author		Andrea Venturi
 * @copyright	Copyright (c) 2009-2010 Andrea Venturi (http://www.andreaventuri.it)
 * @license		http://www.gnu.org/licenses/lgpl.html	GNU Lesser General Public License
 * @version		1.1
*/
class FormBuilder
{
	protected $db;
	protected $errors;
	protected $readonly;
	protected $repeat;
	protected $search;
	protected $selects;
	protected $t;
	protected $values;
	protected $view;
	protected $xml;
	protected $extra;
	protected $uniqid;
	protected $expiration;

	/**
	 * Costruttore della classe
	 * @param	string	$file	il percorso completo del file di definizione del form
	 * @param	array	$values	array con i valori predefiniti per ogni campo del form
	 * @return	void
	 */
	public function __construct($file, $database=null, $values=array())
	{
		$this->db		= $database;
		$this->xml		= simplexml_load_file($file);
		$this->values	= $values;

		$this->uniqid	= md5(uniqid(mt_rand(), true));
		$this->cleanSession();

		$this->errors	= array();
		$this->readonly	= array();
		$this->repeat	= array();
		$this->selects	= array();
		$this->extra	= array();

		// scadenza sessione a 5 minuti
		$this->expiration = 5*60;

		$this->t = new My_Translate();
	}

	/**
	 * Si occupa di eliminare i dati del form obsoleti dalla sessione
	 * @return void
	 */
	protected function cleanSession()
	{
		if(isset($_SESSION[__CLASS__]))
		{
			foreach($_SESSION[__CLASS__] as $key=>$val)
			{
				if( time()>=$val['expire'] ) unset($_SESSION[__CLASS__][$key]);
			}
		}
	}

	public function setXml($xml)
	{
		$this->xml = simplexml_load_file($xml);
	}

	public function getValues()
	{
		return $this->values;
	}

	/**
	 * Sets default values for the elements in the form.
	 * @param	array	$values	array('field_id_1' => 'value_1', 'field_id_2' => 'value_2')
	 * @return	void
	 */
	public function setValues($values)
	{
		$this->values = $values;
	}

	/**
	 * Unsets default values for the elements in the form
	 * @return void
	 */
	public function unsetValues()
	{
		unset($this->values);
	}

	/**
	 * Sets additional values to insert inside <select> elements.
	 * @param	array	$values	options displayed in <select> elements. array('field_id_1' => array('key_1_1'=>'value_1_1'))
	 * @return	void
	 */
	public function setSelects($values)
	{
		$this->selects = $values;
	}

	/**
	 * Imposta l'array con il numero di ripetizioni per ogni pagina del form
	 * @param	array	$repeat	array associativo con "id_pagina" => numero ripetizioni
	 * @return	void
	 */
	public function setRepeat($repeat)
	{
		$this->repeat = $repeat;
	}

	public function setReadOnly($readonly)
	{
		$this->readonly	= $readonly;
	}

	public function getReadOnly()
	{
		return $this->readonly;
	}

	private function loadReadOnly()
	{
		$arr = $this->xml->xpath('//field[readonly=1]');

		foreach($arr as $obj)
		{
			$this->readonly[strval($obj->id)] = 1;
		}
	}

	/**
	 * Sets the attribute "action" in the form
	 * @param	string	$action
	 * @return	void
	 */
	public function setAction($action)
	{
		$this->xml->action = $action;
	}

	/**
	 * Sets the label for an element, given its id
	 * @param	string	$label
	 * @param	int		$id
	 * @return	void
	 */
	public function setLabel($label, $id)
	{
		$res = $this->findId($id);
		if(!empty($res)) $res->label = $label;
	}

	/**
	 * Finds an element (input, select, button, textarea), given its id
	 * @param	string	$id	element's id to retrieve
	 * @return	SimpleXMLElement | false
	 */
	private function findId($id)
	{
		foreach($this->xml->page as $page)
		{
			foreach($page->section as $section)
			{
				$res = $this->scanSection($section, $id);
				if(!empty($res)) return $res;
			}
		}
		return false;
	}

	/**
	 * Recursive function to find an element (input, select, button, textarea), given its id
	 * @param	SimpleXMLElement	$section	SimpleXMLElement to scan
	 * @param	string				$id			element's id to retrieve
	 * @return SimpleXMLElement or false
	 */
	private function scanSection($section, $id)
	{
		foreach($section->field as $field)
		{
			if(strval($field->id) == $id) return $field;
		}

		foreach($section->section as $s2)
		{
			$ret = $this->scanSection($s2, $id);
			if(isset($res) && !$res) return $res;
		}
		return false;
	}

	/**
	 * Returns an array with all the ids of the fields in the form
	 * @return array
	 */
	public function getIds()
	{
		$arr = $this->xml->xpath('//field/id');
		$arr2 = array();
		foreach($arr as $val)
			$arr2[] = strval($val);

		return $arr2;
	}

	/**
	 * Returns an array with all the ids of the fields in the form and values as default
	 * @param $value default value for fill the array
	 * @return array
	 *
	public function getFields($value=0)
	{
		$keys = $this->getIds();

		return array_fill_keys($keys,$value);
	}
*/
	/**
	 * Valids the values posted with the form
	 * @return bool
	 */
	public function isValid()
	{
		// 09/10/2009 MODIFICA faccio il merge tra i campi postati e quelli readonly in sessione
		// mi serve per visualizzare i dati nel form senza ricaricarli da DB
		$this->uniqid = $_POST['uniqid'];

		if(isset($_SESSION[__CLASS__][$this->uniqid]))
		{
			$arr = array_merge($_POST, $_SESSION[__CLASS__][$this->uniqid]);
		}
		else
		{
			$arr = $_POST;
		}

		/* cancella i dati dalla sessione.
			- se è valido guadagno spazio perchè non servono più
			- se non è valido sono pari xkè poi viene ricostruito comunque
		*/
		unset($_SESSION[__CLASS__][$this->uniqid]);

		// imposto i valori dei campi e faccio controlli
		$this->values = $arr;

		$v1 = $this->checkRequired();
		$v2 = $this->validation();

		return ($v1 && $v2);
	}

	/**
	 * Cerca i field con required=1 e controlla che non sia vuoto.
	 * Ckecks all required fields of the form
	 * @return bool
	 */
	protected function checkRequired()
	{
		$arr = $this->xml->xpath('//field[required=1]');

		$valid = true;
		foreach($arr as $obj)
		{
			// se il valore è vuoto ed il campo non è readonly
			if(	$obj->type!='file' && empty($this->values[strval($obj->id)]) && $this->readonly[strval($obj->id)]==0 ||
				$obj->type=='file' && empty($_FILES[strval($obj->id)]) 		 && $this->readonly[strval($obj->id)]==0 )
			{
				$valid = false;
				$this->errors[strval($obj->id)] = 'field_required';
			}
		}

		return $valid;
	}

	/**
	 * Controlla che i field contengano valori conformi a quanto dichiarato
	 * Checks all values in the form against the type of validation defined
	 * @return bool
	 */
	protected function validation()
	{
		$arr = $this->xml->xpath('//field[validation]');

		$valid = true;
		foreach($arr as $obj)
		{
			if( !$this->valid($obj->validation, $this->values[strval($obj->id)],$error) )
			{
				$valid = false;
				$this->errors[strval($obj->id)] = $error;
			}
		}

		return $valid;
	}

	/**
	 * Valida un valore secondo il tipo dichiato
	 * Valids a value against a type
	 * @param $type string
	 * @param $value mixed
	 * @param $error string
	 * @return bool
	 */
	private function valid($type, $value, &$error=null)
	{
		$valid = true;
		if(empty($value)) return $valid;
		switch($type)
		{
			case 'email':
				if(!eregi("^[a-z0-9_\-]+(\.[a-z0-9_\-]+)*@([a-z0-9_\-]+\.)+([a-z]{2,6})$", $value))
				{
					$valid = false;
					$error = 'email_not_valid';
				}
				break;

			case 'number':
				if(!is_numeric($value))
				{
					$valid = false;
					$error = 'numeric_not_valid';
				}
				break;

			case 'date':
				if(!eregi("^[0-9]{1,2}/[0-9]{1,2}/[0-9]{2,4}$", $value))
				{
					$valid = false;
					$error = 'date_not_valid';
				}
				break;

			default:
				if( get_class($type) == 'SimpleXMLElement')
				{
					switch($type->type)
					{
						case 'equalTo':
							$val = $this->values[strval($type->field)];

							if($value!=$val)
							{
								$valid = false;
								$error = 'value_mismatch';
							}
							break;
					}
				}
		}
		return $valid;
	}

	/**
	 * Stampa i controlli javascript per validare i valori del form prima di essere postati
	 * Prints javascript code for validation
	 * @return string
	 */
	private function print_check_js()
	{
		$obb = $this->xml->xpath('//field[required=1]/id');
		$val = $this->xml->xpath('//field[validation]');

		// matrice con gli id dei campi e le validazioni
		$elem = array();
		$mess = array();

		// ciclo i campi obbligatori
		if(is_array($obb))
		{
			foreach($obb as $el)
			{
				$elem[strval($el)][] = 'required: true';
			}
		}

		// ciclo le validazioni
		foreach($val as $item)
		{
			foreach($item->validation as $itm)
			{
				if(isset($itm->type))
				{
					//equalTo: "#password"
					$elem[strval($item->id)][] = $itm->type.': "#'.$itm->field.'"';

					// messaggi personalizzati con traduzioni
					if($itm->type=='gt')
						$mess[strval($item->id)][] = $itm->type.': "Inserire un valore maggiore di \"'.$this->t->_(strval($itm->field)).'\""';

					if($itm->type=='ge')
						$mess[strval($item->id)][] = $itm->type.': "Inserire un valore maggiore o uguale di \"'.$this->t->_(strval($itm->field)).'\""';
				}
				else
				{
					$elem[strval($item->id)][] = $item->validation.': true';
				}
			}
		}

		$rows = array();
		foreach($elem as $key=>$el) $rows[] = '"' . $key . '"' . ': {' . implode(', ', $el) . '}';

		$msgs = array();
		foreach($mess as $key=>$el) $msgs[] = '"' . $key . '"' . ': {' . implode(', ', $el) . '}';

		if(count($rows)>0)
		{
			echo "
					<script type=\"text/javascript\">

					$(function(){

						$('#".$this->xml->id."').validate({
								rules: {
									" . implode(', ', $rows) . "
								},
								messages: {
									" . implode(', ', $msgs) . "
								},
								errorElement: 'span',
								errorClass: 'form_error',
								submitHandler: function(form) {form.submit();}
							});
					});
					</script>
				";
		}
	}

	/**
	 * Prints extra javascript code (e.g. calendar)
	 * @return string
	 */
	private function print_extra_js()
	{
		//if( count($this->extra)>0 )
		//{
			echo "
					<script type=\"text/javascript\">
					<!--
						$(function(){\n";

			$res = $this->xml->xpath('//field[extra="calendar"]');
			if(!empty($res))
			{
				echo	"	$('.calendar').datepicker({
								showOn: 'both',
								buttonImage: 'images/calendar.gif',
								buttonImageOnly: true
							});\n";
			}

			$res = $this->xml->xpath('//field[extra="datemask"]');
			if(!empty($res))
			{
				echo "	$('.datemask').mask('99/99/9999');

						$('.datemask').blur(function(){
									data = $(this).val();
									regex = new RegExp('([0-9]+)\/([0-9]+)\/([0-9]+)','gi');
									data = regex.exec(data);
									if( data != null && data.length==4 )
									{
										if( data[1]<=31 && data[2]<=12 ) return true;
										else
										{
											alert('Data non valida');
											$(this).focus();
										}
									}
						});\n";
			}

			$res = $this->xml->xpath('//field[extra="timemask"]');
			if(!empty($res))
			{
				echo "	$('.timemask').mask('99:99');

						$('.timemask').blur(function(){
									data = $(this).val();
									regex = new RegExp('([0-9]+):([0-9]+)','gi');
									data = regex.exec(data);
									if( data != null && data.length==3 )
									{
										if( data[1]<=23 && data[2]<=59 ) return true;
										else
										{
											alert('Ora non valida');
											$(this).focus();
										}
									}
						});\n";
			}

			$res = $this->xml->xpath('//field[extra="tinymce"]');
			if(!empty($res))
			{
				echo "	$('textarea.tinymce').tinymce({
							// Location of TinyMCE script
							script_url : 'js/tiny_mce/tiny_mce.js',

							// General options
							theme : 'advanced',
							plugins : 'fullscreen,noneditable',
							noneditable_leave_contenteditable : true,

							// Theme options
							theme_advanced_buttons1 : 'bold,italic,underline,|,bullist,numlist,|,outdent,indent,|,undo,redo,|,fullscreen',
							theme_advanced_buttons2 : '',
							theme_advanced_buttons3 : '',
							theme_advanced_toolbar_location : 'top',
							theme_advanced_toolbar_align : 'left',
							theme_advanced_statusbar_location : 'bottom',
							theme_advanced_resizing : true
						});\n";
			}

			echo "		});
					-->
					</script>";
		//}
	}

	private function calendar($obj, $id='')
	{
		if(isset($obj->search))
		{
			$i = 0;
			foreach($obj->search->op as $op)
			{
				$ids[] = $obj->id.'_'.$i;
				$i++;
			}
			$id = implode(', #',$ids);
		}
		else
		{
			//$id = $obj->id;
		}

		// se il campo non è readonly
		if( $this->readonly[strval($obj->id)]!=1 && $obj->readonly!=1)
		{
			$this->extra[] = "	$('#".$id."').datepicker({
										showOn: 'both',
										buttonImage: '/images/calendar.gif',
										buttonImageOnly: true
								});";
		}
	}

	/**
	 * Stampa il form ed eventualmente il codice javascript
	 * Prints the form and javascript code
	 * @param $view		true se il form deve essere stampato in sola lettura
	 * @param $search	true se il form è in modalità ricerca
	 * @return string
	 */
	public function print_form($view=false, $search=false)
	{
		$this->view 	= $view;
		$this->search 	= $search;

		ob_start();

		if(isset($this->xml->method)) $method = $this->xml->method;
		else $method = 'post';

		// onsubmit="return false;"
		echo '<form action="',$this->xml->action,'" method="',$method,'"';

		if(!empty($this->xml->id))
		{
			echo ' id="',$this->xml->id,'" ';
		}
		else
		{
			$this->xml->id = 'form1';
			echo ' id="',$this->xml->id,'" ';
		}

		if(!empty($this->xml->class))	echo ' class="',$this->xml->class,'" ';
		if(!empty($this->xml->enctype))	echo ' enctype="',$this->xml->enctype,'" ';

		echo ">\n";

		$j = 0;
		foreach($this->xml->page as $page)
		{
			$quanti = empty($page->repeat) ? 1 : $page->repeat;

			if(isset( $this->repeat[strval($page->id)] ))
			{
				$quanti = $this->repeat[strval($page->id)];
			}

			if($quanti>0)
			{
				if($quanti==1 && empty($page->prefix))
				{
					$this->build_page($page, ++$j);
				}
				else
				{
					for($i=1;$i<=$quanti;$i++)
					{
						$this->build_page($page, ++$j, array('count'=>$i));
					}
				}
			}
		}

		// un campo nascosto per riconoscere quale form è stato postato, solo quando è in edit
		if(!$this->search)
		{
			echo '<input type="hidden" id="uniqid" name="uniqid" value="'.$this->uniqid.'" />';
		}

		echo "</form>\n";
		echo '<script type="text/javascript">$(".page:visible:last").css("page-break-after","auto");</script>';

		// controllo js degli errori ed extra solo quando si è in edit/add
		// oppure stampa di tinymce se presente in view
		$tiny = $this->xml->xpath('//field[extra="tinymce"]');
		if(!$this->view)
		{
			if(!$this->search) $this->print_check_js();
			$this->print_extra_js();
		}
		elseif(!empty($tiny))
		{
			$this->print_extra_js();
		}

		// se c'è almeno un campo readonly imposto la scadenza della validità
		if(isset($_SESSION[__CLASS__][$this->uniqid]))
		{
			$_SESSION[__CLASS__][$this->uniqid]['expire'] = time() + $this->expiration;
			//$_SESSION[__CLASS__][$this->uniqid]['expire2'] = date(DATETIME_DB,time()+1*60);
		}

		//echo '<pre>', print_r( $_SESSION ,true), '</pre>';
		//echo '@@', date(DATETIME_DB) ,'@@';

		return ob_get_clean();
	}

	private function build_page($page, $id, $param=array())
	{
		$c = !empty($param['count']) ? $param['count'] : 1;

		echo '<div      id="page_',$page->id,$c,'" class="fieldset page '.$page->class.'" >', "\n";
		//echo '<div      id="page_',$id,'" class="fieldset page '.$page->class.'" >', "\n";
		//echo '<fieldset id="page_',$id,'" class="page '.$page->class.'">', "\n";

		// print caption if present
		if(!empty($page->caption))
		{
			// substitute placeholder in caption
			$caption = $page->caption;
			if(!empty($param['count']))
			{
				$caption = sprintf($caption, $param['count']);
			}
			//echo '<legend><span>',$caption,'</span></legend>', "\n";
			echo '<span class="legend">',$caption,'</span>', "\n";
		}

		$prefix = !empty($page->prefix) ? $page->prefix . '['.$param['count'].']' : '';

		$j = 1;
		foreach($page->section as $section) $this->build_section($section, $id.'_'.$j++, $prefix);

		//echo "</fieldset>\n";
		echo "</div>\n";

		//echo "<div style=\"page-break-before: always\"><span style=\"visibility: hidden;\">-</span></div>";
		//echo "<br style=\"page-break-before: always\" />";
		//echo "<hr style=\"page-break-before: always; display:none\" />";
	}

	private function build_section($section, $id, $prefix='', $param=array())
	{
		$class = !empty($section->class) ? ' class="'.$section->class.'" ' : '';

		if(!empty($section->id))
			$id = strval($section->id);
		else
			$id = 'section_'.$id;

		//echo '<fieldset id="',$id,'" ',$class,'>', "\n";
		echo '<div      id="',$id,'" class="fieldset ',$section->class,'" >', "\n";

		if(!empty($section->caption))
		{
			$caption = $section->caption;
			if(!empty($param['count']))
			{
				$caption = sprintf($caption, $param['count']);
			}
			//echo '<legend>',$section->caption,'</legend>', "\n";
			echo '<span class="legend">',$caption,'</span>', "\n";
		}

		// print fields
		foreach($section->field as $field)
		{
			if($this->search)
				$this->build_field_search($field,$prefix);
			else
				$this->build_field($field,$prefix);
		}

		// print children sections
		$j = 1;
		foreach($section->section as $s2)
		{
			$quanti = empty($s2->repeat) ? 1 : $s2->repeat;

			$loc = str_replace(']','',str_replace('[','_',$prefix)) . '_' . strval($s2->id);

			if(isset( $this->repeat[$loc] )) $quanti = $this->repeat[$loc];

			if($quanti>0)
			{
				if($quanti==1 && empty($s2->prefix))
				{
					//$this->build_page($s2, ++$j);
					$this->build_section($s2, $id.'_'.$j++, $prefix);
				}
				else
				{
					for($i=1;$i<=$quanti;$i++)
					{
						//$this->build_page($s2, ++$j, array('count'=>$i));
						$this->build_section($s2, $id.'_'.$j++, $prefix.'['.strval($s2->prefix).']['.$i.']', array('count'=>$i));
					}
				}
			}
		}

		//echo "</fieldset>\n";
		echo "<div class=\"float_none clear_both\"></div></div>\n";
	}

	/**
	 *
	 * @param	SimpleXMLElement	$field
	 * @param	string				$prefix	prefisso per l'id e il nome del campo
	 * @return	void
	 */
	private function build_field($field, $prefix='')
	{
		// insert the prefix if present
		if(!empty($prefix))
		{
			$id		= str_replace(']','',str_replace('[','_',$prefix)) . '_' . strval($field->id);
			$name	= $prefix . '[' . strval($field->id) . ']';
		}
		else
		{
			$id		= strval($field->id);
			$name	= $id;
		}

		$extra = '';
		if(isset($field->extra))
		{
			//$this->calendar($field, $id);
			$extra = $field->extra;
		}

		if(isset($field->readonly))
		{
			$this->readonly[$id] = 1;
		}

		//$edit = !$this->view && !$this->readonly[$id];
		$edit = !$this->view && empty($this->readonly[$id]);

		// prendo la label se impostata, altrimenti l'id
		$label = isset($field->label) ? strval($field->label) : strval($field->id);

		// traduzione
		if(isset($this->t)) $label = $this->t->_($label);

		// asterisco per campo obbligatorio
		$req = $field->required==1 ? ' *' : '';
		$label = $label.$req;

		/* scrive il div quando:
		 * - è un bottone e si è in edit/add
		 * - non è un hidden nè un bottone
		 *
		 * serve per non stampare il div quando si è in view e c'è un campo hidden
		 */
		$show_div = ($field->type=='button' && $edit) || ($field->type!='hidden' && $field->type!='button');
		if( $show_div )
		{
			if(!empty($field->class))
				$class = 'class="_div '.$field->class.'"';
			else
				$class = 'class="_div"';

			echo '<div id="',str_replace(':','_',$id),'_div" ',$class,' >';

			switch($field->type)
			{
				case 'radio':
					echo '<label>',$label,'</label>';
					break;

				case 'button':
				case 'multicheck':
					break;

				default:
					echo '<label for="',$id,'">',$label,'</label>';
			}
		}

		switch ($field->type)
		{
			case 'button':

				if($edit)
				{
					$this->print_button($field, $id, $label);
				}
				break;

			case 'textarea':

				if($edit)
				{
					$this->print_textarea($field, $id, $name, $this->values[$id], $extra);
				}
				else
				{
					if($extra=='tinymce')
					{
						echo '<textarea id="',$id,'" name="',$name,'" class="',$extra,'" >';
						if( isset($this->values[$id]) )
						{
							echo '<div class="mceNonEditable">' . $this->values[$id] . '</div>';
						}
						echo '</textarea>';
					}
					else
					{
						echo '<div class="textarea" id="',$id,'">';
						if( isset($this->values[$id]) ) echo $this->values[$id];
						echo '</div>';
					}
				}
				break;

			case 'select':

				if($edit)
				{
					$this->print_select($field, $id, $name, $this->values[$id]);
				}
				else
				{
					echo '<div class="select" id="',$id,'">';

					if (!empty($field->lookup))
					{
						$q1 =	'SELECT ' .  $field->lookup->id . ' AS id, ' . $field->lookup->caption . ' AS caption ' .
								'FROM ' . $field->lookup->table;
						$res = $this->db->fetchAll($q1);

						foreach ($res as $row)
						{
							if( isset($this->values[$id]) && $this->values[$id] == $row['id'] )
							{
								if( intval($field->lookup->showId)==1 )
								{
									echo $row['id'] . ' - ' . $row['caption'];
								}
								else
								{
									echo $row['caption'];
								}
							}
						}
					}

					if (!empty($field->values->opt))
					{
						foreach($field->values->opt as $opt)
						{
							if( isset($this->values[$id]) && $this->values[$id] == $opt->value )
							{
								echo $this->t->_(strval($opt->value));
							}
						}
					}

					// scorre i valori dinamici da config
					if(isset($this->selects[$id]))
					{
						foreach($this->selects[$id] as $key=>$val)
						{
							if( isset($this->values[$id]) && $this->values[$id] == $key )
							{
								echo $this->t->_($val);
							}
						}
					}
					echo '</div>';
				}
				break;

			case 'radio':

				if (!empty($field->values->opt))
				{
					$value = isset($this->values[$id]) ? $this->values[$id] : strval($field->value);

					echo '<div class="inner_radio">';
					foreach($field->values->opt as $opt)
					{
						//$opt_id = $id.'_'.$opt->id.'_'.$opt->value;
						$opt_id = $id.'_'.$opt->value;
						echo '<label for="',$opt_id,'">',$this->t->_(strval($opt->label)),'</label>';
						echo '<input type="',$field->type,'" id="',$opt_id,'" name="',$name,'" class="radio" ';
						echo 'value="',$opt->value,'"';

						if( isset($value) && $value == strval($opt->value) )
						{
							echo ' checked="checked" ';
						}

						if( !$edit ) echo ' disabled="disabled" ';

						echo ' />';
					}
					echo '</div>';
				}
				break;

			case 'checkbox':

				echo '<input type="',$field->type,'" id="',$id,'" name="',$name,'"';
				echo ' class="checkbox" ';
				echo 'value="',$field->value,'"';

				if( isset($this->values[$id]) && $this->values[$id] == $field->value )
				{
					echo ' checked="checked" ';
				}

				if( !$edit ) echo ' disabled="disabled" ';

				echo ' />';
				break;

			case 'multicheck':

				if (!empty($field->lookup))
				{
					$q1 =	'SELECT ' .  $field->lookup->id . ' AS id, ' . $field->lookup->caption . ' AS caption ' .
							'FROM ' . $field->lookup->table . ' ';

					if(!empty($field->lookup->where))
					{
						$q1 .= 'WHERE ' . $field->lookup->where;
					}
					if(!empty($field->lookup->orderby))
					{
						$q1 .= ' ORDER BY ' . $field->lookup->orderby;
					}
					else
					{
						$q1 .= ' ORDER BY id ASC';
					}

					$res = $this->db->fetchAll($q1);
					foreach ($res as $row)
					{
						// MODIFICA ANDREA 26/06/2009
						// mostro solo le righe con i check selezionati
						$id2 	= $id . '_' . $row['id'];

						$values = isset($this->values[$id2]) ? $this->values[$id2] : false;

						if( $values || !$this->view)
						{
							$name2 	= $name . '[' . $row['id'] . ']';

							echo '<div id="',$id2,'_div" class="multicheck">';

							echo '<input type="checkbox" id="',$id2,'" name="',$name2,'"';
							echo ' class="checkbox" value="1" ';

							if( $values == 1 ) echo ' checked="checked" ';

							if( !$edit ) echo ' disabled="disabled" ';

							echo ' />';

							echo '<span>', My_Utility::htmlutf($row['caption']), '</span>';

							echo '</div>';
						}
					}
				}
				break;

			case 'span':
				$this->print_span($field, $id);
				break;

			default:

				$value = isset($this->values[$id]) ? $this->values[$id] : strval($field->value);

				if($edit)
				{
					$this->print_input($field, $id, $name, $value, $extra);
				}
				else
				{
					if($field->type!='hidden')
					{
						echo '<div class="input" id="',$id,'">';
						if(isset($value)) echo $value;
						echo '</div>';
					}
				}
		}

		// 09/10/2009 MODIFICA se il campo è in readonly lo metto in sessione
		if(!$edit)
		{
			$_SESSION[__CLASS__][$this->uniqid][$id] = strval($this->values[$id]);
		}

		// elemento che contiene la descrizione degli errori
		if( !empty($this->errors[strval($id)]) )
		{
			$error = isset($this->t) ? $this->t->_($this->errors[strval($id)]) : $this->errors[strval($id)];
			echo '<span class="form_error">',$error,'</span>';
		}

		if( $show_div )
		{
			echo '</div>', "\n";
		}
	}

	private function build_field_search($field, $prefix='')
	{
		$count = count($field->search->op);

		if($count>0 || $field->type=='button' || $field->type=='span')
		{
			// insert the prefix if present
			if(!empty($prefix))
			{
				$id = $prefix . '_' . strval($field->id);
			}
			else
			{
				$id = strval($field->id);
			}

			$extra = '';
			if(isset($field->extra))
			{
				//$this->calendar($field, $id);
				$extra = $field->extra;
			}

			// prendo la label se impostata, altrimenti l'id
			$label = isset($field->label) ? strval($field->label) : strval($field->id);

			// traduzione
			if(isset($this->t)) $label = $this->t->_($label);

			if(!empty($field->class))
				$class = 'class="_div '.$field->class.'"';
			else
				$class = 'class="_div"';

			//echo '<div id="',$id,'_div">';
			echo '<div id="',str_replace(':','_',$id),'_div" ',$class,' >';

			if($field->type!='button')
				echo '<label>',$label,'</label>';

			//echo '<label for="',$id,'">',$label,'</label>';

			switch ($field->type)
			{
				case 'button':

					$this->print_button($field, $id, $label);
					break;

				case 'textarea':

					$this->print_textarea($field, $id, $id, $this->values[$id]);
					break;

				case 'select':

					$i = 0;
					foreach($field->search->op as $op)
					{
						echo '<div>';
						$this->print_op(strval($op), strval($field->id), $i);

						$this->print_select($field, $id.'_'.$i, $id.'['.$i.'][val]', $this->values[$id][$i]['val']);

						$i++;

						if($i<$count)
						{
							echo '<select class="frm_operator_con" name="'.$id.'['.$i.'][con]">
									<option value="and"',
										$this->values[$id][$i]['con'] == 'and' ? ' selected="selected" ' : ''
									,'>AND</option>
									<option value="or"',
										$this->values[$id][$i]['con'] == 'or' ? ' selected="selected" ' : ''
									,'>OR</option>
									</select>';
						}
						echo '</div>';
					}
					break;

				case 'radio':

					if (!empty($field->values->opt))
					{
						$i = 0;

						$value = !empty($this->values[$id][$i]['val']) ? $this->values[$id][$i]['val'] : strval($field->value);

						$op = strval($field->search->op[$i]);
						echo '<div>';
						$this->print_op(strval($op), strval($field->id), $i);

						echo '<div class="inner_radio">';
						foreach($field->values->opt as $opt)
						{
							$opt_id = $id.'_'.$opt->value;

							echo '<label for="',$opt_id,'">',$opt->label,'</label>';
							echo '<input type="',$field->type,'" id="',$opt_id,'" name="',$id.'['.$i.'][val]','" class="radio" ';
							echo 'value="',$opt->value,'"';

							if( isset($value) && $value == $opt->value )
							{
								echo ' checked="checked" ';
							}

							echo ' />';
						}
						echo '</div>';
						echo '</div>';
					}
					break;

				case 'checkbox':

					$i = 0;
					$op = strval($field->search->op[$i]);
					echo '<div>';
					$this->print_op(strval($op), strval($field->id), $i);

					echo '<input type="',$field->type,'" id="',$id,'" name="',$id.'['.$i.'][val]','"';
					echo ' class="checkbox" ';
					echo 'value="',$field->value,'"';

					if( isset($this->values[$id][$i]['val']) && $this->values[$id][$i]['val'] == $field->value )
					{
						echo ' checked="checked" ';
					}

					echo ' />';
					echo '</div>';
					break;

				case 'span':
					$this->print_span($field, $id);
					break;

				default:

					$i = 0;
					foreach($field->search->op as $op)
					{
						echo '<div>';
						$this->print_op(strval($op), strval($field->id), $i);

						$this->print_input($field, $id.'_'.$i, $id.'['.$i.'][val]', $this->values[$id][$i]['val'], $extra);

						$i++;

						if($i<$count)
						{
							echo '<select class="frm_operator_con" name="'.$id.'['.$i.'][con]">
									<option value="and"',
										$this->values[$id][$i]['con'] == 'and' ? ' selected="selected" ' : ''
									,'>AND</option>
									<option value="or"',
										$this->values[$id][$i]['con'] == 'or' ? ' selected="selected" ' : ''
									,'>OR</option>
									</select>';
						}
						echo '</div>';
					}
			}

			// elemento che contiene la descrizione degli errori
			if( !empty($this->errors[strval($id)]) )
			{
				$error = isset($this->t) ? $this->t->_($this->errors[strval($id)]) : $this->errors[strval($id)];
				echo '<span class="form_error">',$error,'</span>';
			}

			echo '</div>', "\n";
		}
	}

	private function print_op($op, $id, $i)
	{
		$all_num['eq'] = '=';
		$all_num['gt'] = '>';
		$all_num['ge'] = '>=';
		$all_num['lt'] = '<';
		$all_num['le'] = '<=';

		$all_str['like'] 		= 'LIKE';
		$all_str['not_like']	= 'NOT LIKE';

		echo '<select class="frm_operator" name="',$id,'[',$i,'][op]">';

		$op = split(',',$op);

		foreach($op as $oper)
		{
			switch(trim($oper))
			{
				case 'all_num':

					foreach($all_num as $key=>$val)
					{
						echo '<option value="'.$key.'"';
						if($this->values[$id][$i]['op']==$key) echo ' selected="selected" ';
						echo '>'.My_Utility::htmlutf($val).'</option>';
					}
					break;

				case 'all_str':

					foreach($all_str as $key=>$val)
					{
						echo '<option value="'.$key.'"';
						if($this->values[$id][$i]['op']==$key) echo ' selected="selected" ';
						echo '>'.My_Utility::htmlutf($val).'</option>';
					}
					break;

				default:

					if(array_key_exists($oper, $all_str))
						$arr = $all_str;
					else
						$arr = $all_num;

					echo '<option value="'.$oper.'"';
					if($this->values[$id][$i]['op']==$oper) echo ' selected="selected" ';
					echo '>'.My_Utility::htmlutf($arr[$oper]).'</option>';
			}
		}

		echo '</select>';
	}

	private function print_button($field, $id, $label)
	{
		echo '<button id="',$id,'" type="',$field->type2,'">', $label, '</button>';
	}

	private function print_textarea($field, $id, $name, $value, $class='')
	{
		echo '<textarea id="',$id,'" name="',$name,'" class="',$class,'" >';
		if( isset($value) ) echo $value;
		echo '</textarea>';
	}

	private function print_input($field, $id, $name, $value, $class='')
	{
		echo '<input type="',$field->type,'" id="',$id,'" name="',$name,'"';
		if( isset($class) ) echo ' class="',$class,'" ';
		if( isset($value) ) echo ' value="',$value,'" ';
		echo ' />';
	}

	/**
	 * Prints a select (combobox) element
	 *
	 * @param	SimpleXMLElement	$field
	 * @param	int					$id			id of the element
	 * @param	string				$name		name of the element
	 * @param	mixed				$default	default value of the element
	 * @return	string
	 */
	private function print_select($field, $id, $name, $default)
	{
		$t = $this->t;

		$default = !empty($default) ? $default : strval($field->value);

		echo '<select id="',$id,'" name="',$name,'">';

		// scorre i valori fissi da xml
		if (!empty($field->values->opt))
		{
			foreach($field->values->opt as $opt)
			{
				echo '<option value="',$opt->value,'"';

				if( isset($default) && $default == $opt->value )
				{
					echo ' selected="selected" ';
				}
				echo '>', $t->_(strval($opt->label)), '</option>';
			}
		}

		// scorre i valori della query
		if (!empty($field->lookup))
		{
			$q1 =	'SELECT ' .  $field->lookup->id . ' AS id, ' . $field->lookup->caption . ' AS caption ' .
					'FROM ' . $field->lookup->table . ' ';

			if(!empty($field->lookup->where))
			{
				$q1 .= 'WHERE ' . $field->lookup->where;
			}
			if(!empty($field->lookup->orderby))
			{
				$q1 .= ' ORDER BY ' . $field->lookup->orderby;
			}
			else
			{
				$q1 .= ' ORDER BY id ASC';
			}

			$res = $this->db->fetchAll($q1);
			foreach ($res as $row)
			{
				echo '<option value="',$row['id'],'"';

				if( isset($default) && $default == $row['id'] )
				{
					echo ' selected="selected" ';
				}

				echo '>';

				if( intval($field->lookup->showId)==1 )
				{
					echo $row['id'] . ' - ' . $t->_($row['caption']);
				}
				else
				{
					echo $t->_($row['caption']);
				}

				echo '</option>';
			}
		}

		// scorre i valori dinamici da config
		if(isset($this->selects[$id]))
		{
			foreach($this->selects[$id] as $key=>$val)
			{
				echo '<option value="',$key,'"';

				if( isset($default) && $default == $key )
				{
					echo ' selected="selected" ';
				}
				echo '>', $t->_($val), '</option>';
			}
		}
		echo '</select>';
	}

	private function print_span($field, $id)
	{
		if(isset($field->class)) $class = 'class="'.$field->class.'"';

		echo '<span id="',$id,'"'.$class.'>';
		echo $field->value;
		echo '</span>';
	}
}

class My_Utility
{
	/**
	 * Tenta la conversione di una stringa UTF8 in entities HTML.
	 * @param	string	$str	stringa da codificare
	 * @return	string	stringa codificata in HTML
	 */
	public static function htmlutf($str)
	{
		$ret = '';

		$ret = htmlentities($str,ENT_COMPAT,'UTF-8');
		if(strlen($ret)<1) $ret = htmlentities($str);

		return $ret;
	}
}

class My_Translate
{
	public function _($str)
	{
		return $str;
	}
}
